package com.nfe.guardian.shared.model.resources;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.security.InvalidParameterException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.io.input.CloseShieldInputStream;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.xml.sax.SAXException;

@org.springframework.data.mongodb.core.mapping.Document
public abstract class NFE implements NFEGuardianResource {
	
	public static final String XML_VERSION = "1.0";
	
	protected String id;
	
	private TransformerFactory factory;
	private Transformer transformer;
	private static DocumentBuilderFactory documentBuilderFactory;	
	
	static {		
		documentBuilderFactory = DocumentBuilderFactory.newInstance();
	    documentBuilderFactory.setNamespaceAware(true);
	}
		
	protected NFE() throws TransformerConfigurationException, ParserConfigurationException {
		factory = TransformerFactory.newInstance();
		transformer = factory.newTransformer();			
	}
	
	@Override
	public abstract String getVersao();	
	
	public String getId() {
		return id;
	}

	@Override
	public String toXML() throws Exception {					
		return toXML(false);		
	}	
		
	@Override
	public String toXML(boolean isFragment) throws Exception {		
		Writer writer = new StringWriter();		
		toXML(isFragment, writer);
		return writer.toString();
	}
	
	@Override
	public void toXML(Writer writer) throws Exception {
		toXML(false, writer);
		
	}

	@Override
	public void toXML(boolean isFragment, Writer writer) throws Exception {
		JAXBContext context = getContext();			
		
		Marshaller marshaller = context.createMarshaller();		
		marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
		marshaller.setProperty(Marshaller.JAXB_FRAGMENT, isFragment);
		
		transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, isFragment ? "yes" : "no");
		
		DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
		Document document = documentBuilder.newDocument();				
		marshaller.marshal(this, document);
		Writer fakeWriter = new StringWriter();
		writer.write(normalize(document, fakeWriter));		
		
	}

	private NFEGuardianResource fromXML(InputStream xml) throws Exception {

		JAXBContext context = getContext();
		
		Unmarshaller unmarshaller = context.createUnmarshaller();		
		
		NFE nfe = (NFE) unmarshaller.unmarshal(xml);		
		return nfe;
				
	}	
	
	private String normalize(Document document, Writer writer) throws TransformerException {		
		document.getDocumentElement().removeAttribute("xmlns:ns4");
        document.getDocumentElement().removeAttribute("xmlns:ns3");
        document.getDocumentElement().removeAttribute("xmlns:ns2");
		
        transformer.transform(new DOMSource(document.getDocumentElement()), new StreamResult(writer));        
        return writer.toString()
        		.replace("<ns2:", "<")
        		.replace("</ns2:", "</")
        		.replace(" xmlns=\"http://www.w3.org/2000/09/xmldsig#\"", "")
        		.replace("xmlns:ns2=\"http://www.portalfiscal.inf.br/nfe\"", "xmlns=\"http://www.portalfiscal.inf.br/nfe\"")
        		.replace("<Signature>", "<Signature xmlns=\"http://www.w3.org/2000/09/xmldsig#\">");        
	}
	
	private JAXBContext getContext() throws JAXBException, UnsupportedOperationException {
		JAXBContext context = null;
		
		if ("2.00".equals(getVersao()))      context = JAXBContext.newInstance("com.nfe.guardian.shared.model.nfe.v200.xml");
		else if ("1.10".equals(getVersao())) context = JAXBContext.newInstance("com.nfe.guardian.shared.model.nfe.v110.xml");
		
		if (context == null) throw new UnsupportedOperationException("A versão "+getVersao()+" não suportada");
		
		return context;
	}	

	@SuppressWarnings("unused")
	private static NFE newInstance(InputStream in) throws Exception {
		
		if (in.available() <= 0) throw new InvalidParameterException("The inputstream is empty");
		
		InputStream markedInput = in.markSupported() ? in : new BufferedInputStream(in);
		CloseShieldInputStream shieldInputStream = new CloseShieldInputStream(markedInput);	
		// TODO Aprender a usuar o mark e corrigir isso, não deve ser fixo, ou mudar a implementação
		markedInput.mark(10000);	
		
		final String versao = getVersao(shieldInputStream); 
	    
	    NFE nfe = new NFE() {
			
			@Override
			public String getVersao() {
				return versao;
			}
			
		};	
		
		
		markedInput.reset();		
		return (NFE) nfe.fromXML(markedInput);
	}

	private static String getVersao(CloseShieldInputStream shieldInputStream) throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();		
	    Document document = documentBuilder.parse(shieldInputStream);  	    
	    NamedNodeMap nodes = document.getDocumentElement().getAttributes();
	    final String versao = nodes.getNamedItem("versao").getTextContent();
		return versao;
	}
	
}
