package com.nfe.guardian.shared.model.resources;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashMap; 
import java.util.Map;

import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;


public final class NFEGuardianResourceFactory {
	
	private static Map<Class<? extends NFEGuardianResource>, Method> cachedMethods = new HashMap<Class<? extends NFEGuardianResource>, Method>();
	
	
	public static <TIPO extends NFEGuardianResource> TIPO newInstance(Class<? extends NFEGuardianResource> clazz, File xml) throws FileNotFoundException, SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		return newInstance(clazz, new FileInputStream(xml));
	}
	
	public static <TIPO extends NFEGuardianResource> TIPO newInstance(Class<? extends NFEGuardianResource> clazz, String xml) throws UnsupportedEncodingException, SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		return newInstance(clazz, new ByteArrayInputStream(xml.getBytes("UTF-8")));
	}
	
	public static <TIPO extends NFEGuardianResource> TIPO newInstance(Class<? extends NFEGuardianResource> clazz, DOMSource xml) throws UnsupportedEncodingException, SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException, TransformerConfigurationException, TransformerException, TransformerFactoryConfigurationError {
		Writer writer = new StringWriter();  		  
		StreamResult result = new StreamResult(writer);  		  
		TransformerFactory.newInstance().newTransformer().transform(xml, result);
		return newInstance(clazz, writer.toString());
		  		
	}
		
	@SuppressWarnings("unchecked")
	public static <TIPO extends NFEGuardianResource> TIPO newInstance(Class<? extends NFEGuardianResource> clazz, InputStream xml) throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		
		Method method = cachedMethods.get(clazz);
		if (method == null) {			
			try {
				method = clazz.getDeclaredMethod("newInstance", InputStream.class);
			} catch (NoSuchMethodException e) {
				throw new NoSuchMethodException("Method newInstance(java.io.InputStream) not found in resource class "+clazz.getName()+". Did you forgot to create it?");
			}
			
			if (!Modifier.isStatic(method.getModifiers())) throw new NoSuchMethodException("Method "+clazz.getName()+".newInstance(java.io.InputStream) is not static");
			
			method.setAccessible(true);
			cachedMethods.put(clazz, method);
		}
		
		
		return (TIPO) method.invoke(null, xml);				
	}
	
}
