package com.nfe.guardian.shared.model.resources;

import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.math.BigInteger;
import java.util.Collection;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.mapping.Document;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;



/**
 * @author tiago
 * @version 1.0
 * @created 29-abr-2012 11:43:29
 */

@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "user", propOrder = {
    "email",
    "password",
    "authorities"
})
@Document
public class User implements UserDetails, NFEGuardianResource {
		
	private static final long serialVersionUID = -8143003272286909174L;
		
	@Id
	@XmlTransient
	private BigInteger id;		
	private String email;
	private String password;
	
	@XmlElement(name="grupos",type=UserGroup.class)	
	private Collection<GrantedAuthority> authorities;

	public User() {}
	
	public User(String email, String password) {
		super();
		this.email = email;
		this.password = password;
	}

	public User(String username, String password,
			Collection<GrantedAuthority> authorities) {
		this.email = username;
		this.password = password;
		this.authorities = authorities;
	}
	
	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	
	public BigInteger getId() {
		return id;
	}
		
	@Override
	public Collection<GrantedAuthority> getAuthorities(){
		return authorities;
	}

	@Override
	public String getPassword(){
		return password;
	}

	@Override
	public String getUsername(){
		return email;
	}

	@Override
	public boolean isAccountNonExpired(){
		return true;
	}

	@Override
	public boolean isAccountNonLocked(){
		return true;
	}

	@Override
	public boolean isCredentialsNonExpired(){
		return true;
	}

	@Override
	public boolean isEnabled(){
		return true;
	}
	
	@XmlAttribute
	@Override
	public String getVersao() {		
		return "1.00";
	}	
	
	@Override
	public void toXML(Writer writer) throws Exception {
		toXML(false, writer);		
	}

	@Override
	public void toXML(boolean isFragment, Writer writer) throws Exception {
		JAXBContext context = JAXBContext.newInstance(User.class);		
		Marshaller marshaller = context.createMarshaller();
		marshaller.setProperty(Marshaller.JAXB_FRAGMENT, isFragment);
		marshaller.marshal(this, writer);				
	}
	
	@Override
	public String toXML() throws Exception {
		return toXML(false);
	}
	
	@Override
	public String toXML(boolean isFragment) throws Exception {		
		Writer writer = new StringWriter();
		toXML(isFragment, writer);
		return writer.toString();
	}


	@Override
	public String toString() {
		return "User [id=" + id + ", email=" + email + ", password=" + password
				+ ", versao="+getVersao()+"]";
	}	
	
	@SuppressWarnings("unused")
	private static User newInstance(InputStream in) throws JAXBException {
		JAXBContext context = JAXBContext.newInstance(User.class);
		return (User) context.createUnmarshaller().unmarshal(in);
	}
	
}