package com.nfe.guardian.shared.model.response;

import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.nfe.guardian.shared.model.resources.NFEGuardianResource;
import com.nfe.guardian.shared.model.resources.NFEGuardianResourceFactory;
import com.nfe.guardian.shared.model.resources.NFEGuardianResourceType;

public class NFEGuardianResponse<T extends NFEGuardianResource> extends ArrayList<T> implements NFEGuardianResource {	

	private static final long serialVersionUID = -278764959402372905L;	
	private NFEGuardianResourceType resourceType;
	
	private static DocumentBuilderFactory documentBuilderFactory;	
	static {		
		documentBuilderFactory = DocumentBuilderFactory.newInstance();
	    documentBuilderFactory.setNamespaceAware(true);
	}
	
	public NFEGuardianResponse(NFEGuardianResourceType resourceType) {
		this.resourceType = resourceType;		
	}
	
	private NFEGuardianResponse() {}
	
	@Override
	public String getVersao() {		
		return "1.00";
	}	
		
	@Override
	public String toXML() throws Exception {
		return toXML(false);
	}

	@Override
	public String toXML(boolean isFragment) throws Exception {
		Writer writer = new StringWriter();
		toXML(isFragment, writer);
		return writer.toString();
	}
	
	@Override
	public void toXML(Writer writer) throws Exception {
		toXML(false, writer);		
	}

	@Override
	public void toXML(boolean isFragment, Writer writer) throws Exception {				
		if (!isFragment) 
			writer.write("<?xml version=\"1.0\" " +
					"encoding=\"UTF-8\" standalone=\"yes\" ?>");
		
		writer.write("<resources type=\""+resourceType+"\">");
						
		Iterator<T> iterator = this.iterator();
		
		while (iterator.hasNext()) {
			NFEGuardianResource resource = (T) iterator.next();			
			writer.write(resource.toXML(true));
		}
				
		writer.write("</resources>");		
	}
	
	@Override
	public boolean add(T item) {
		if (!resourceType.getResourceClass()
				.isAssignableFrom(item.getClass())) return false;
		return super.add(item);
	}

	@SuppressWarnings("unused")
	private static NFEGuardianResponse<? extends NFEGuardianResource> newInstance(InputStream xml) 
			throws Exception {
		NFEGuardianResponse<NFEGuardianResource> resources = new NFEGuardianResponse<NFEGuardianResource>();
		
		Document document = documentBuilderFactory.newDocumentBuilder().parse(xml); 
		Element root = document.getDocumentElement();
		
		resources.resourceType = NFEGuardianResourceType.fromName(root.getAttribute("type"));
		
		NodeList resourceNodes = root.getChildNodes();
				
		for (int i=0; i < resourceNodes.getLength(); i++) {
			Node resourceNode = resourceNodes.item(i);
			resources.add(NFEGuardianResourceFactory
					.newInstance(
							resources.resourceType.getResourceClass(), 
							new DOMSource(resourceNode)));
		}
						
		return resources;		
	}	

}
