package com.nfe.guardian.shared.model;

import static org.junit.Assert.*;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.Writer;
import java.util.List;

import org.junit.Test;

import com.nfe.guardian.shared.model.resources.NFE;
import com.nfe.guardian.shared.model.resources.NFEGuardianError;
import com.nfe.guardian.shared.model.resources.NFEGuardianResource;
import com.nfe.guardian.shared.model.resources.NFEGuardianResourceFactory;
import com.nfe.guardian.shared.model.resources.NFEGuardianResourceType;
import com.nfe.guardian.shared.model.resources.User;
import com.nfe.guardian.shared.model.response.NFEGuardianResponse;

public class ResourceTester {
	
	@Test
	public void readAndWriteNfe() throws Exception {
		InputStream in = new FileInputStream(new File("./testFiles/42120285098929000158550070000025451220137122.xml"));
		
		NFE nfe = NFEGuardianResourceFactory.newInstance(NFE.class, in);
		
		assertEquals("2.00", nfe.getVersao());
				
		Writer writer = new FileWriter(new File("./testFiles/responses/v1/nfeFromXml.xml"));
		nfe.toXML(writer);
		writer.flush();
		writer.close();
			
	}
	
	@Test
	public void writeUser() throws Exception {
		User user = new User();
		user.setEmail("tiago@teste.com.br");
		user.setPassword("1234");
		String xml = user.toXML();
		assertNotNull(xml);
		
		Writer writer = new FileWriter(new File("./testFiles/responses/v1/user.xml"));
		writer.write(xml);
		writer.flush();
		writer.close();
			
	}
	
	@Test
	public void readUser() throws Exception {
		InputStream in = new FileInputStream(new File("./testFiles/responses/v1/user.xml"));
		
		User user = NFEGuardianResourceFactory.newInstance(User.class, in);
					
		assertEquals("1234", user.getPassword());
			
	}
	
	@Test
	public void writeUserList() throws Exception {
		NFEGuardianResponse<NFEGuardianResource> users = new NFEGuardianResponse<NFEGuardianResource>(NFEGuardianResourceType.USER);
		
		for (int i = 0; i < 4; i++) {
			User user = new User();
			user.setEmail("tiago@teste.com.br");
			user.setPassword("1234");			
			users.add(user);
		}
		
		NFE nfe = new NFE() {
			
			@Override
			public String getVersao() {				
				return null;
			}

			@Override
			public String getId() {				
				return null;
			}
		};
		
		assertFalse(users.add(nfe));
				
		String xml = users.toXML();
		assertNotNull(xml);
				
		
		System.out.println(xml);
		
		Writer writer = new FileWriter(new File("./testFiles/responses/v1/userList.xml"));
		writer.write(xml);
		writer.flush();
		writer.close();
			
	}
	
	@Test
	public void readUserList() throws Exception {
		File f = new File("./testFiles/responses/v1/userList.xml");
		
		List<User> users = NFEGuardianResourceFactory.newInstance(NFEGuardianResponse.class, f);
		
		assertNotNull(users);
		
		System.out.println(users.toString());
			
	}
	
	@Test
	public void writeNFEList() throws Exception {
		File nfePath = new File("./testFiles");
		
		File[] files = nfePath.listFiles(new FileFilter() {
			
			@Override
			public boolean accept(File pathname) {
				if (pathname.isDirectory()) return false;
				return pathname.getName().endsWith(".xml");
			}
		});
			
		NFEGuardianResponse<NFE> nfes = new NFEGuardianResponse<NFE>(NFEGuardianResourceType.NFe);
		for (File xml : files) {
			NFE nfe = NFEGuardianResourceFactory.newInstance(NFE.class, xml);
			assertEquals("NFe"+xml.getName().replace(".xml", ""), nfe.getId());
			nfes.add(nfe);
		}
				
		Writer writer = new FileWriter(new File("./testFiles/responses/v1/nfeList.xml"));
		nfes.toXML(writer);
		writer.flush();
		writer.close();
		
	}
	
	@Test
	public void readNFEList() throws Exception {
		File f = new File("./testFiles/responses/v1/nfeList.xml");
		
		List<NFE> nfes = NFEGuardianResourceFactory.newInstance(NFEGuardianResponse.class, f);
		
		assertNotNull(nfes);
		
		System.out.println(nfes.toString());
			
	}
	
	@Test
	public void writeAndReadError() throws Exception {
		Writer writer = new FileWriter(new File("./testFiles/responses/v1/error.xml"));
		
		NFEGuardianError error = new NFEGuardianError(404, "Not Found", "O recurso não foi encontrado");
		
		NFEGuardianResponse<NFEGuardianError> response = new NFEGuardianResponse<NFEGuardianError>(NFEGuardianResourceType.ERROR);
		response.add(error);
		
		response.toXML(writer);
		
		assertNotNull(writer);
		
		writer.flush();
		writer.close();
		
		InputStream in = new FileInputStream(new File("./testFiles/responses/v1/error.xml"));
		
		NFEGuardianResponse<NFEGuardianError> errorRead = NFEGuardianResourceFactory.newInstance(NFEGuardianResponse.class, in);
		assertNotNull(errorRead);
		
		System.out.println(errorRead.toString());
			
	}
	
	@Test
	public void introspectionTest() throws IntrospectionException {
		BeanInfo bi = Introspector.getBeanInfo(User.class);
		PropertyDescriptor[] propertyDescriptors = bi.getPropertyDescriptors();
		for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
			System.out.printf("Property[name=%s, type=%s, readMethod=%s, writeMethod=%s, editorClass=%s]\n",
					propertyDescriptor.getName(),
					propertyDescriptor.getPropertyType(),
					propertyDescriptor.getReadMethod(),
					propertyDescriptor.getWriteMethod(),
					propertyDescriptor.getPropertyEditorClass());
		}
	}
	
	
}
